/*******************************************************************************
**	Permedia II RAMDAC driver
********************************************************************************
**
 * Permedia2 framebuffer driver.
 * Copyright (c) 1998-1999 Ilario Nardinocchi (nardinoc@CS.UniBO.IT)
 * Driver used for port under KGI license with permission of Ilario 
 * Nardinocchi.
 * Copyright (c) 1999 Andreas Beck - port to KGI
**
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
*/

#include <kgi/maintainers.h>
#define	MAINTAINER	Andreas_Beck
#define	DRIVER_NAME	"Permedia 2 fb-derived ramdac driver"
#define	DRIVER_REV	"$Revision: 1.4 $"

#include <kgi/module.h>
#include <kgi/kgi_commands.h>

#include <3dlabs/pm2free.h>

static struct kgi_ramdac ramdac = 
{
	"3dLabs",			/* Manufacturer		*/
	"Permedia2",			/* Model		*/
	GM_8BIT | GM_16BIT |
	GM_24BIT | GM_32BIT,		/* Supported modes	*/
	0,			 	/* Flags		*/
	{ 0xFFFF, 0xFFFF },		/* Maximum resolution	*/
	0,				/* priv pointer         */
	DAC_64BIT_PB,  			/* Device type		*/
	{       0,  230000000 },	/* Normal mode		*/
	{       0,         0 },		/* 2xclk mode		*/
	{
		{	  0,         0 },	/* KGIGT_TEXT16	*/
		{	  0,         0 },	/* KGIGT_TEXT32	*/
		{         0,         0 },	/* KGIGT_1BIT	*/
		{	  0,         0 },	/* KGIGT_4BIT	*/
		{         0, 230000000 },	/* KGIGT_8BIT	*/	
		{	  0,         0 },	/* KGIGT_15BIT	*/
		{ 	  0, 170000000 },	/* KGIGT_16BIT	*/
		{         0, 110000000 },	/* KGIGT_24BIT	*/
		{	  0, 110000000 }	/* KGIGT_32BIT	*/
	}
};

/* GGI interface */

int kgim_ramdac_init(struct kgi_card *card, struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " rev. " DRIVER_REV);
	return EOK;
}

void kgim_ramdac_done(struct kgi_display *dpy)
{
	NOTICE(DRIVER_NAME " removed.");
}

int kgim_ramdac_check_mode (struct kgi_display *dpy, struct kgi_mode *mode,
	enum kgi_tm_cmd *cmd)
{
	kgi_uint newclock = mode->dclk;
  
	switch (mode->request.graphtype) {

	case KGIGT_8BIT:
	case KGIGT_16BIT:
	case KGIGT_24BIT:
	case KGIGT_32BIT:
		break;
  
	default:
  		DEBUG2("couldn't handle graphtype %i", 
		       mode->request.graphtype);
		return -E(RAMDAC,MODE_CHECK);
	}

	switch (*cmd) {

	case CMD_PROPOSE:
		mode->lclk.mul = mode->clk.mul = 1;
		mode->lclk.div = mode->clk.div = 1;
		return EOK;

	case CMD_LOWER:

		if ((newclock < ramdac.clock[mode->request.graphtype].min) ||
			(newclock < ramdac.normal.min)) {

			DEBUG2("DCLK too low (%i Hz).", newclock);
			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if (ramdac.clock[mode->request.graphtype].max < newclock) {

			newclock = ramdac.clock[mode->request.graphtype].max;
		}
		mode->dclk = newclock;
		return EOK;

	case CMD_RAISE:
		if ((newclock > ramdac.clock[mode->request.graphtype].max) ||
			(newclock > ramdac.normal.max)) {

			DEBUG2("DCLK too high (%i Hz).", newclock);
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		if (newclock < ramdac.clock[mode->request.graphtype].min) {

			newclock = ramdac.clock[mode->request.graphtype].min;
		}

		if (newclock < ramdac.normal.min) {

			newclock = ramdac.normal.min;
		}

		mode->dclk = newclock;
		return EOK;

     case CMD_CHECK:
		if ((mode->lclk.mul != 1) || (mode->lclk.div != 1) ||
		    (mode->clk.mul != 1) || (mode->clk.div != 1)) {

			return -E(RAMDAC,MODE_CHECK);
		}

		if ((mode->dclk < ramdac.clock[mode->request.graphtype].min) ||
		    (mode->dclk < ramdac.normal.min)) {

			return -E(RAMDAC,CLK_LOW_LIMIT);
		}

		if ((mode->dclk > ramdac.clock[mode->request.graphtype].max) ||
		    (mode->dclk > ramdac.normal.max)) {
			
			return -E(RAMDAC,CLK_HIGH_LIMIT);
		}

		return EOK;

	default:
		INTERNAL_ERROR;
		return -E(RAMDAC,BOGUS);
	}
}

struct chipset_private;

/* called from DAC driver
 */
static void PM2_set_color(struct chipset_private* p, unsigned char regno,
			unsigned char r, unsigned char g, unsigned char b) {

	WAIT_FIFO(p, 4);
	pm2_WR(p, PM2R_RD_PALETTE_WRITE_ADDRESS, regno);
	DEFW();
	pm2_WR(p, PM2R_RD_PALETTE_DATA, r);
	DEFW();
	pm2_WR(p, PM2R_RD_PALETTE_DATA, g);
	DEFW();
	pm2_WR(p, PM2R_RD_PALETTE_DATA, b);
	DEFW();
}


static void dac_set_clut(struct kgi_display *dpy, kgi_clut *clut,
			kgi_uint first, kgi_uint cnt)
{
	GET_PRIVATE;

	ASSERT(cnt);

	while(cnt--) {
		PM2_set_color(private, first,
				clut->data[first].r>>8,
				clut->data[first].g>>8,
				clut->data[first].b>>8);
		first++;
	}
}

void kgim_ramdac_set_mode(struct kgi_display *dpy, struct kgi_mode *mode)
{
/*        pm2_RDAC_WR(private, PM2I_RD_COLOR_MODE, PM2F_RD_COLOR_MODE_RGB| */
/*			PM2F_RD_GUI_ACTIVE|clrmode); */
	dpy->set_clut = dac_set_clut;
}

int kgim_ramdac_command(struct kgi_graphic *graph, kgi_uint cmd, kgi_uint arg)
{
	/* This may set 
	 * -  Hardware cursor
	 * -  CLUT (or gamma correction table)
	 */
	kgi_clut clut;

	switch (cmd) { 

	case RAMDAC_SETCLUT:
		clut.size = 256;
		clut.data = (kgi_color *)(graph->io_buf);
		dac_set_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;

#if 0
	case RAMDAC_GETCLUT:
		clut.size = 256;
		clut.data = (kgi_color *)(graph->io_buf);
		graph->io_res = graph->io_buf;
		dac_get_clut(graph->dev.dpy, &clut, 0, 256);
		return 0;
#endif
	}
  	return -E(RAMDAC, NOSUP_ALWAYS_CANT);  /* The ramdac can't do this */
}
